/* ggisyna - program to display sound graphically
   Copyright (C) 1997       Paul Francis Harrison
   Copyright (C) 1997-1998  Marcus Sundberg

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>

#include "syna.h"

static int sndfd, mixdev;


int
sndSource(int source)
{
    int sndfd = open(MIXDEV, O_WRONLY);
    
    switch (source) {
    case SYNSND_LINE:
        source=SOUND_MASK_LINE;
        break;
    case SYNSND_CD:
        source=SOUND_MASK_CD;
        break;
    case SYNSND_SYNTH:
        source=SOUND_MASK_SYNTH;
        break;
    case SYNSND_MIC:
        source=SOUND_MASK_MIC;
        break;
    default:
        return -1;
    }

    if (sndfd == -1) {
        warning("opening " MIXDEV);
        mixdev=0;
    } else {
        int temp = source;
        attemptNoDie(ioctl(sndfd,SOUND_MIXER_WRITE_RECSRC,&temp),"writing to mixer");
        close(sndfd);
    }
    return 0;
}


int
sndStart(int *pipes)
{
    int format, stereo, fragment, fqc;
    int pid;
    short *data;
    
    switch (pid=fork()) {
    case 0:
        close(pipes[0]);
        
        attempt(sndfd = open(SNDDEV, O_RDONLY),"opening " SNDDEV);
        
        format = AFMT_S16_LE;
        fqc = FREQUENCY;
        stereo = 1;
        fragment = 0x00080000 + (m-overlap+1); /* 8 fragments of size 
						  2*(2^(m-overlap+1)) bytes */
        
        attemptNoDie(ioctl(sndfd,SNDCTL_DSP_SETFRAGMENT,&fragment),"setting fragment");
        attempt(ioctl(sndfd,SNDCTL_DSP_SETFMT,&format),"setting format");
        if (format != AFMT_S16_LE) error("setting format (2)");
        attempt(ioctl(sndfd,SNDCTL_DSP_STEREO,&stereo),"setting stereo");
        attemptNoDie(ioctl(sndfd,SNDCTL_DSP_SPEED,&fqc),"setting frequency");
	
        data = malloc(SAMPLES*2*sizeof(short));
	
        for(;;) {
            attempt(read(sndfd,data,SAMPLES*4),"reading from dsp");
            if (write(pipes[1],data,SAMPLES*4) != SAMPLES*4) break;
        }
	
        close(pipes[1]);
        free(data);
        exit(0);
        break;
    case -1:
        return -1;
        break;
    default:
        return pid;
        break;
    }
}


void
mixUp(int source)
{
    int mixfd, tmp;
    
    switch (source) {
    case SYNSND_MAIN:
        source=SOUND_MIXER_VOLUME;
        break;
    case SYNSND_LINE:
        source=SOUND_MIXER_LINE;
        break;
    case SYNSND_CD:
        source=SOUND_MIXER_CD;
        break;
    case SYNSND_SYNTH:
        source=SOUND_MIXER_SYNTH;
        break;
    case SYNSND_MIC:
        source=SOUND_MIXER_MIC;
        break;
    default:
        return;
    }
    
    if ((mixfd = open(MIXDEV, O_RDWR)) > -1) {
        ioctl(mixfd,MIXER_READ(source),&tmp);
        tmp+=MIXINT+(MIXINT<<8);
        ioctl(mixfd,MIXER_WRITE(source),&tmp);
        close(mixfd);
    }
}

void
mixDown(int source)
{
    int mixfd, tmp;
    
    switch (source) {
    case SYNSND_MAIN:
        source=SOUND_MIXER_VOLUME;
        break;
    case SYNSND_LINE:
        source=SOUND_MIXER_LINE;
        break;
    case SYNSND_CD:
        source=SOUND_MIXER_CD;
        break;
    case SYNSND_SYNTH:
        source=SOUND_MIXER_SYNTH;
        break;
    case SYNSND_MIC:
        source=SOUND_MIXER_MIC;
        break;
    default:
        return;
    }

    if ((mixfd = open(MIXDEV, O_RDWR)) > -1) {
        ioctl(mixfd,MIXER_READ(source),&tmp);
        tmp-=MIXINT+(MIXINT<<8);
        if (tmp < 0)
            tmp=0;
        ioctl(mixfd,MIXER_WRITE(source),&tmp);
        close(mixfd);
    }
}


